﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Problems.Scheduling.Encodings;

namespace HeuristicLab.Problems.Scheduling.Encodings {
  [Item("JobShop Scheduling Problem Creator", "Represents the generalized form of creators for JobShop Scheduling Problems.")]
  [StorableClass]
  public abstract class JSSPCreator<T> : SchedulingCreator where T : JSSPEncoding{
    [StorableConstructor]
    protected JSSPCreator(bool deserializing) : base(deserializing) { }
    protected JSSPCreator(JSSPCreator<T> original, Cloner cloner)
      : base(original, cloner) {
    }
    
    public ILookupParameter<T> SchedulingSolutionParameter {
      get { return (ILookupParameter<T>)Parameters["SchedulingSolution"]; }
    }

    public JSSPCreator() : base (){
      Parameters.Add(new LookupParameter<T>("SchedulingSolution", "The new scheduling solution."));
    }

    public override IOperation Apply() {
      SchedulingSolutionParameter.ActualValue = CreateSolution();
      return base.Apply();
    } 


    protected abstract T CreateSolution();
    
  }
}
