﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.Scheduling.Interfaces;

namespace HeuristicLab.Problems.Scheduling.Encodings {
  [Item("JSSPCrossover", "A scheduling crossover operation.")]
  [StorableClass]
  public abstract class JSSPCrossover : SchedulingOperator, IJSSPCrossover {
    #region IJSSPCrossover Members
    public ILookupParameter<ItemArray<JSSPEncoding>> ParentsParameter {
      get { return (ScopeTreeLookupParameter<JSSPEncoding>)Parameters["Parents"]; }
    }

    public ILookupParameter<JSSPEncoding> ChildParameter {
      get { return (ILookupParameter<JSSPEncoding>)Parameters["Child"]; }
    }
    #endregion

    [StorableConstructor]
    protected JSSPCrossover(bool deserializing) : base(deserializing) { }
    protected JSSPCrossover(JSSPCrossover original, Cloner cloner)
      : base(original, cloner) {
    }
    public JSSPCrossover()
      : base() {
      Parameters.Add(new ScopeTreeLookupParameter<JSSPEncoding>("Parents", "The parent permutations which should be crossed."));
      ParentsParameter.ActualName = "SchedulingSolution";
      Parameters.Add(new LookupParameter<JSSPEncoding>("Child", "The child permutation resulting from the crossover."));
      ChildParameter.ActualName = "SchedulingSolution";
    }
  }
}
