﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Data;
using HeuristicLab.Parameters;

namespace HeuristicLab.Problems.Scheduling.Encodings {
  [Item("JobShop Scheduling Encoding", "An abstract class that holds all necessary data for solving the standard JobShop Scheduling problem.")]
  [StorableClass]
  public abstract class JSSPEncoding : ParameterizedNamedItem {
    [StorableConstructor]
    protected JSSPEncoding(bool deserializing) : base(deserializing) { }
    protected JSSPEncoding(JSSPEncoding original, Cloner cloner)
      : base(original, cloner) {
        this.Quality = cloner.Clone(original.Quality);
    }

    public JSSPEncoding() : base() {
      Quality = new DoubleValue();
    }

    [Storable]
    public DoubleValue Quality {
      get;
      set;
    }

    public abstract Schedule ToSchedule(IntValue nrOfResources, ItemList<JSSPJob> jobs, IRandom random);

  }
}
