﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Data;
using HeuristicLab.Problems.Scheduling.Encodings;

namespace HeuristicLab.Problems.Scheduling.Encodings.JobSequenceMatrix {
  [Item("Job Sequencing Matrix Encoding", "Represents a solution for a standard JobShop Scheduling Problem.")]
  [StorableClass]
  public class JSMEncoding : JSSPEncoding{
    [StorableConstructor]
    protected JSMEncoding(bool deserializing) : base(deserializing) { }
    protected JSMEncoding(JSMEncoding original, Cloner cloner)
      : base(original, cloner) {
        this.JobSequenceMatrix = cloner.Clone(original.JobSequenceMatrix);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new JSMEncoding(this, cloner);
    }

    [Storable]
    public ItemList<Permutation> JobSequenceMatrix { get; set; }

    public JSMEncoding() : base() {
      JobSequenceMatrix = new ItemList<Permutation>();
    }

    public override Schedule ToSchedule(IntValue nrOfResources, ItemList<JSSPJob> jobs, IRandom random) {
      JSMDecoder decoder = new JSMDecoder(nrOfResources, random);
      return decoder.CreateScheduleFromJSM (jobs, JobSequenceMatrix);
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append("[ ");

      foreach (Permutation p in JobSequenceMatrix) {
        sb.Append(p.ToString() + " \n");
      }

      sb.Append("]");
      return sb.ToString();
    }
  }
}
