﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Data;
using HeuristicLab.Parameters;

namespace HeuristicLab.Problems.Scheduling {
  [Item("Job Class", "Represents a composition of tasks that require processing in a job shop scheduling problem.")]
  [StorableClass]
  public class JSSPJob : ParameterizedNamedItem {
    [StorableConstructor]
    protected JSSPJob(bool deserializing) : base(deserializing) { }
    protected JSSPJob(JSSPJob original, Cloner cloner)
      : base(original, cloner) {
      this.JobIndex = cloner.Clone(original.JobIndex);
      this.Tasks = cloner.Clone(original.Tasks);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new JSSPJob(this, cloner);
    }

    #region Parameter Properties
    public ValueParameter<IntValue> JobIndexParameter {
      get { return (ValueParameter<IntValue>)Parameters["JobIndex"]; }
    }

    public ValueParameter<ItemList<Task>> TasksParameter {
      get { return (ValueParameter<ItemList<Task>>)Parameters["Tasks"]; }
    }
    #endregion

    #region Properties
    public IntValue JobIndex {
      get { return JobIndexParameter.Value; }
      set { JobIndexParameter.Value = value; }
    }
    public ItemList<Task> Tasks {
      get { return TasksParameter.Value; }
      set { TasksParameter.Value = value; }
    }
    #endregion


    public JSSPJob(IntValue ji) {
      Parameters.Add(new ValueParameter<IntValue>("JobIndex", "The index of the job in the associated JSSP-Instance.", new IntValue()));
      Parameters.Add(new ValueParameter<ItemList<Task>>("Tasks", "Taskdata defining duration, start-time and resource assignment of the tasks.", new ItemList<Task>()));

      JobIndex = ji;
      Tasks = new ItemList<Task>();
    }


    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append("Job" + JobIndex + " [ ");
      foreach (Task t in Tasks) {
        sb.Append(t.ToString () + " ");
      }
      sb.Append("]");
      return sb.ToString();
    }

  }
}
