﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;

namespace HeuristicLab.Problems.Scheduling {
  [Item("Job Shop Scheduling Task Class", "Represents smallest unit for a job shop scheduling problem.")]
  [StorableClass]
  public class JSSPTask : Task {
    [StorableConstructor]
    protected JSSPTask(bool deserializing) : base(deserializing) { }
    protected JSSPTask(JSSPTask original, Cloner cloner)
      : base(original, cloner) {
      this.Job = cloner.Clone(original.Job);
      this.PreviousTask = cloner.Clone(original.PreviousTask);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new JSSPTask(this, cloner);
    }



    public JSSPJob Job { get; set; }
    public Task PreviousTask { get; set; }

    public JSSPTask (IntValue taskNr, IntValue resNr, DoubleValue duration, JSSPJob j) : base (taskNr, resNr, duration) {
      Job = j;
    }
  }
}
