﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;

//ATTENTION! This class may have lots of parallels with the HeuristicLab.Problems.Scheduling.JSSPJob class.

namespace HeuristicLab.Problems.Scheduling {
  [Item("Resource Class", "Represents a resource used in scheduling problems.")]
  [StorableClass]
  public class Resource : ParameterizedNamedItem {
    [StorableConstructor]
    protected Resource(bool deserializing) : base(deserializing) { }
    protected Resource(Resource original, Cloner cloner)
      : base(original, cloner) {
      this.ResourceIndex = cloner.Clone(original.ResourceIndex);
      this.Tasks = cloner.Clone(original.Tasks);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new Resource(this, cloner);
    }

    #region Parameter Properties
    public ValueParameter<IntValue> ResourceIndexParameter {
      get { return (ValueParameter<IntValue>)Parameters["ResourceIndex"]; }
    }
    public ValueParameter<ItemList<Task>> TasksParameter {
      get { return (ValueParameter<ItemList<Task>>)Parameters["Tasks"]; }
    }
    #endregion

    #region Properties
    public IntValue ResourceIndex {
      get { return ResourceIndexParameter.Value; }
      set { ResourceIndexParameter.Value = value; }
    }
    public ItemList<Task> Tasks {
      get { return TasksParameter.Value; }
      set { TasksParameter.Value = value; }
    }
    public DoubleValue TotalDuration {
      get {
        if (Tasks.Count > 0)
          return new DoubleValue(Tasks[Tasks.Count - 1].EndTime.Value);
        else
          return new DoubleValue (0);
      }
    }
    #endregion


    public Resource(IntValue ri) {
      Parameters.Add(new ValueParameter<IntValue>("ResourceIndex", "The index of the resource in the associated Scheduling Problem.", new IntValue()));
      Parameters.Add(new ValueParameter<ItemList<Task>>("Tasks", "Taskdata defining duration, start-time and resource assignment of the tasks.", new ItemList<Task>()));

      ResourceIndex = ri;
      Tasks = new ItemList<Task>();
    }


    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append("Job" + ResourceIndex + " [ ");
      foreach (Task t in Tasks) {
        sb.Append(t.ToString() + " ");
      }
      sb.Append("]");
      return sb.ToString();
    }

  }
}
