﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Operators;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Parameters;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.Scheduling {
  [Item("SchedulingOperator", "A scheduling operator.")]
  [StorableClass]
  public abstract class SchedulingOperator : SingleSuccessorOperator{
    [StorableConstructor]
    protected SchedulingOperator(bool deserializing) : base(deserializing) { }
    protected SchedulingOperator(SchedulingOperator original, Cloner cloner)
      : base(original, cloner) {
    }

    #region Parameter Properties
    public ILookupParameter<IntValue> NrOfResourcesParameter {
      get { return (ILookupParameter<IntValue>)Parameters["NrOfResources"]; }
    }
    public ILookupParameter<ItemList<JSSPJob>> JobsParameter {
      get { return (ILookupParameter<ItemList<JSSPJob>>)Parameters["Jobs"]; }
    }
    #endregion

    #region Properties
    public IntValue NrOfResources {
      get { return NrOfResourcesParameter.ActualValue; }
    }
    public ItemList<JSSPJob> Jobs {
      get { return JobsParameter.ActualValue; }
    }
    #endregion

    public SchedulingOperator()
      : base() {
      Parameters.Add(new LookupParameter<IntValue>("NrOfResources", "Number of Resources used in this JSSP-Instance."));
      Parameters.Add(new LookupParameter<ItemList<JSSPJob>>("Jobs", "Jobdata defining the precedence relationships and the duration of the tasks in this JSSP-Instance."));
    }


  }
}
