﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Common;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;



namespace HeuristicLab.Problems.Scheduling {
  [Item("Task Class", "Represents the smallest unit of schedule definitions.")]
  [StorableClass]
  public class Task : Item {
    [StorableConstructor]
    protected Task(bool deserializing) : base(deserializing) { }
    protected Task(Task original, Cloner cloner)
      : base(original, cloner) {
      this.ResourceNr = cloner.Clone(original.ResourceNr);
      this.Duration = cloner.Clone(original.Duration);
      this.StartTime = cloner.Clone(original.StartTime);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new Task(this, cloner);
    }

    public Task () {

    }

    public Task (IntValue taskNr, IntValue resNr, DoubleValue duration) {
      Duration = duration;
      ResourceNr = resNr;
      TaskNr = taskNr;
      StartTime = new DoubleValue (0);
      IsScheduled = new BoolValue (false);
    }


    public BoolValue IsScheduled { get; set; }
    public IntValue TaskNr { get; set; }
    public IntValue ResourceNr { get; set; }
    public DoubleValue Duration { get; set; }
    public DoubleValue StartTime { get; set; }
    public DoubleValue EndTime { 
      get { 
        return new DoubleValue (Duration.Value + StartTime.Value); 
      } 
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append("[");
      sb.Append(String.Format ("{0,4}", ResourceNr));
      sb.Append(";");
      sb.Append(String.Format ("{0,4}", Duration));
      sb.Append("]");
      return sb.ToString();
    }
  }
}
